var Connection, Debug, EventEmitter, Login7Payload, MessageIO, PreloginPayload, STATE, Socket, SqlBatchPayload, TYPE, TokenStreamParser;
var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
  for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
  function ctor() { this.constructor = child; }
  ctor.prototype = parent.prototype;
  child.prototype = new ctor;
  child.__super__ = parent.prototype;
  return child;
};
require('buffertools');
Debug = require('./debug');
EventEmitter = require('events').EventEmitter;
TYPE = require('./packet').TYPE;
PreloginPayload = require('./prelogin-payload');
Login7Payload = require('./login7-payload');
SqlBatchPayload = require('./sqlbatch-payload');
MessageIO = require('./message-io');
Socket = require('net').Socket;
TokenStreamParser = require('./token/token-stream-parser').Parser;
STATE = {
  INITIAL: 0,
  SENT_PRELOGIN: 1,
  SENT_LOGIN7: 2,
  LOGGED_IN: 3,
  SENT_CLIENT_REQUEST: 4,
  SENT_ATTENTION: 5,
  FINAL: 6
};
Connection = (function() {
  __extends(Connection, EventEmitter);
  function Connection(server, userName, password, options, callback) {
    var _base, _base2;
    this.server = server;
    this.userName = userName;
    this.password = password;
    this.options = options;
    this.startRequest = __bind(this.startRequest, this);
    this.eventPacket = __bind(this.eventPacket, this);
    this.eventTimeout = __bind(this.eventTimeout, this);
    this.eventError = __bind(this.eventError, this);
    this.eventEnd = __bind(this.eventEnd, this);
    this.eventConnect = __bind(this.eventConnect, this);
    this.eventClose = __bind(this.eventClose, this);
    this.options || (this.options = {});
    (_base = this.options).port || (_base.port = 1433);
    (_base2 = this.options).timeout || (_base2.timeout = 10 * 1000);
    this.loggedIn = false;
    this.state = STATE.INITIAL;
    this.debug = new Debug(this.options.debug);
    this.debug.on('debug', __bind(function(message) {
      if (this.state !== STATE.FINAL) {
        return this.emit('debug', message);
      }
    }, this));
    this.messagePayloadBuffer = new Buffer(0);
    this.createTokenStreamParser();
    this.connection = new Socket({});
    this.connection.setTimeout(options.timeout);
    this.connection.connect(this.options.port, this.server);
    this.connection.addListener('close', this.eventClose);
    this.connection.addListener('connect', this.eventConnect);
    this.connection.addListener('end', this.eventEnd);
    this.connection.addListener('error', this.eventError);
    this.connection.addListener('timeout', this.eventTimeout);
    this.messageIo = new MessageIO(this.connection, this.debug);
    this.messageIo.on('packet', this.eventPacket);
    this.startRequest('connect/login', callback);
    this.packetBuffer = new Buffer(0);
  }
  Connection.prototype.execSql = function(request) {
    var payload;
    this.sqlRequest = request;
    this.startRequest('execSql', request.callback);
    payload = new SqlBatchPayload(request.sqlText);
    this.state = STATE.SENT_CLIENT_REQUEST;
    this.messageIo.sendMessage(TYPE.SQL_BATCH, payload.data);
    return this.debug.payload(payload.toString('  '));
  };
  Connection.prototype.eventClose = function(hadError) {
    this.emit('closed');
    return this.debug.log("connection close, hadError:" + hadError);
  };
  Connection.prototype.eventConnect = function() {
    this.debug.log('connected');
    return this.sendPreLoginPacket();
  };
  Connection.prototype.eventEnd = function() {
    return this.debug.log('end');
  };
  Connection.prototype.eventError = function(exception) {
    this.fatalError(exception);
    return this.connection.destroy();
  };
  Connection.prototype.eventTimeout = function() {
    this.emit('timeout');
    this.fatalError('timeout');
    return this.connection.destroy();
  };
  Connection.prototype.eventPacket = function(packet) {
    switch (this.state) {
      case STATE.SENT_PRELOGIN:
        return this.buildMessage(packet, this.processPreloginResponse);
      case STATE.SENT_LOGIN7:
      case STATE.SENT_CLIENT_REQUEST:
        return this.tokenStreamParser.addBuffer(packet.data());
      default:
        return this.fatalError("Unexpected packet in state " + this.state + ": packet type " + (packet.type()));
    }
  };
  Connection.prototype.buildMessage = function(packet, payloadProcessFunction) {
    this.messagePayloadBuffer = new Buffer(this.messagePayloadBuffer.concat(packet.data()));
    if (packet.isLast()) {
      payloadProcessFunction.call(this);
      return this.messagePayloadBuffer = new Buffer(0);
    }
  };
  Connection.prototype.processPreloginResponse = function() {
    var preloginPayload;
    preloginPayload = new PreloginPayload(this.messagePayloadBuffer);
    this.debug.payload(preloginPayload.toString('  '));
    return this.sendLogin7Packet();
  };
  Connection.prototype.createTokenStreamParser = function() {
    this.tokenStreamParser = new TokenStreamParser(this.debug);
    this.tokenStreamParser.on('loginack', __bind(function(token) {
      return this.loggedIn = true;
    }, this));
    this.tokenStreamParser.on('infoMessage', __bind(function(token) {
      return this.emit('infoMessage', token);
    }, this));
    this.tokenStreamParser.on('errorMessage', __bind(function(token) {
      this.activeRequest.error = token.message;
      return this.emit('errorMessage', token);
    }, this));
    this.tokenStreamParser.on('packetSizeChange', __bind(function(token) {
      return this.messageIo.packetSize(token.newValue);
    }, this));
    this.tokenStreamParser.on('databaseChange', __bind(function(token) {
      this._database = token.newValue;
      return this.emit('databaseChange', this._database);
    }, this));
    this.tokenStreamParser.on('languageChange', __bind(function(token) {
      this._language = token.newValue;
      return this.emit('languageChange', this._language);
    }, this));
    this.tokenStreamParser.on('charsetChange', __bind(function(token) {
      this._charset = token.newValue;
      return this.emit('charsetChange', this._charset);
    }, this));
    this.tokenStreamParser.on('columnMetadata', __bind(function(token) {
      return this.sqlRequest.emit('columnMetadata', token.columns);
    }, this));
    this.tokenStreamParser.on('row', __bind(function(token) {
      return this.sqlRequest.emit('row', token.columns);
    }, this));
    this.tokenStreamParser.on('returnStatus', __bind(function(token) {
      return this.procReturnStatusValue = token.value;
    }, this));
    this.tokenStreamParser.on('error', __bind(function(token) {
      return this.fatalError(token.error);
    }, this));
    this.tokenStreamParser.on('done', __bind(function(token) {
      var state;
      state = this.state;
      if (this.loggedIn) {
        this.state = STATE.LOGGED_IN;
      }
      if (state === STATE.SENT_LOGIN7) {
        return this.activeRequest.callback(this.activeRequest.error, this.loggedIn);
      } else {
        return this.activeRequest.callback(this.activeRequest.error, token.rowCount);
      }
    }, this));
    return this.tokenStreamParser.on('doneProc', __bind(function(token) {
      this.state = STATE.LOGGED_IN;
      this.activeRequest.callback(this.activeRequest.error, this.procReturnStatusValue);
      return this.procReturnStatusValue = void 0;
    }, this));
  };
  Connection.prototype.startRequest = function(requestName, callback) {
    return this.activeRequest = {
      requestName: requestName,
      info: {
        infos: [],
        errors: [],
        envChanges: []
      },
      callback: callback
    };
  };
  Connection.prototype.sendPreLoginPacket = function() {
    var payload;
    payload = new PreloginPayload();
    this.messageIo.sendMessage(TYPE.PRELOGIN, payload.data);
    this.debug.payload(payload.toString('  '));
    return this.state = STATE.SENT_PRELOGIN;
  };
  Connection.prototype.sendLogin7Packet = function() {
    var loginData, payload;
    loginData = {
      userName: this.userName,
      password: this.password,
      database: this.options.database
    };
    payload = new Login7Payload(loginData);
    this.messageIo.sendMessage(TYPE.LOGIN7, payload.data);
    this.debug.payload(payload.toString('  '));
    return this.state = STATE.SENT_LOGIN7;
  };
  Connection.prototype.fatalError = function(message) {
    this.debug.log("FATAL ERROR: " + message);
    this.close();
    return this.emit('fatal', message);
  };
  Connection.prototype.database = function() {
    return this._database;
  };
  Connection.prototype.language = function() {
    return this._language;
  };
  Connection.prototype.charset = function() {
    return this._charset;
  };
  Connection.prototype.close = function() {
    this.connection.end();
    return this.state = STATE.FINAL;
  };
  return Connection;
})();
module.exports = Connection;