var DEFAULT_PACKET_SIZE, EventEmitter, MessageIO, Packet, isPacketComplete, packetLength;
var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
  for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
  function ctor() { this.constructor = child; }
  ctor.prototype = parent.prototype;
  child.prototype = new ctor;
  child.__super__ = parent.prototype;
  return child;
};
require('buffertools');
EventEmitter = require('events').EventEmitter;
isPacketComplete = require('./packet').isPacketComplete;
packetLength = require('./packet').packetLength;
Packet = require('./packet').Packet;
DEFAULT_PACKET_SIZE = 4 * 1024;
MessageIO = (function() {
  __extends(MessageIO, EventEmitter);
  function MessageIO(connection, debug) {
    this.connection = connection;
    this.debug = debug;
    this.sendPacket = __bind(this.sendPacket, this);
    this.eventData = __bind(this.eventData, this);
    this._packetSize = DEFAULT_PACKET_SIZE;
    this.connection.addListener('data', this.eventData);
    this.packetBuffer = new Buffer(0);
    this.payloadBuffer = new Buffer(0);
  }
  MessageIO.prototype.eventData = function(data) {
    var length, packet, _results;
    this.packetBuffer = new Buffer(this.packetBuffer.concat(data));
    _results = [];
    while (isPacketComplete(this.packetBuffer)) {
      length = packetLength(this.packetBuffer);
      packet = new Packet(this.packetBuffer.slice(0, length));
      this.logPacket('Received', packet);
      this.emit('packet', packet);
      _results.push(this.packetBuffer = new Buffer(this.packetBuffer.slice(length)));
    }
    return _results;
  };
  MessageIO.prototype.packetSize = function(packetSize) {
    if (arguments.length > 0) {
      this.debug.log("Packet size changed from " + this._packetSize + " to " + packetSize);
      this._packetSize = packetSize;
    }
    return this._packetSize;
  };
  MessageIO.prototype.sendMessage = function(packetType, payload) {
    var numberOfPackets, packet, packetNumber, packetPayload, payloadEnd, payloadStart, _ref, _results;
    numberOfPackets = (Math.floor((payload.length - 1) / this._packetSize)) + 1;
    _results = [];
    for (packetNumber = 0, _ref = numberOfPackets - 1; 0 <= _ref ? packetNumber <= _ref : packetNumber >= _ref; 0 <= _ref ? packetNumber++ : packetNumber--) {
      payloadStart = packetNumber * this._packetSize;
      if (packetNumber < numberOfPackets - 1) {
        payloadEnd = payloadStart + this._packetSize;
      } else {
        payloadEnd = payload.length;
      }
      packetPayload = payload.slice(payloadStart, payloadEnd);
      packet = new Packet(packetType);
      packet.last(packetNumber === numberOfPackets - 1);
      packet.packetId(packetNumber);
      packet.addData(packetPayload);
      _results.push(this.sendPacket(packet));
    }
    return _results;
  };
  MessageIO.prototype.sendPacket = function(packet) {
    this.logPacket('Sent', packet);
    return this.connection.write(packet.buffer);
  };
  MessageIO.prototype.logPacket = function(direction, packet) {
    this.debug.packet(direction, packet);
    return this.debug.data(packet);
  };
  return MessageIO;
})();
module.exports = MessageIO;