var ReadableTrackingBuffer, buffertools;
buffertools = require('buffertools');
/*
  A Buffer-like class that tracks position.

  As values are read, the position advances by the size of the read data.
  When reading, if the read would pass the end of the buffer, null is returned.
*/
ReadableTrackingBuffer = (function() {
  function ReadableTrackingBuffer(buffer, encoding) {
    this.buffer = buffer;
    this.encoding = encoding;
    if (!buffer) {
      this.buffer = new Buffer(0);
      this.encoding = void 0;
    }
    this.encoding || (this.encoding = 'utf8');
    this.position = 0;
  }
  ReadableTrackingBuffer.prototype.add = function(buffer) {
    this.buffer = this.buffer.slice(this.position).concat(buffer);
    return this.position = 0;
  };
  ReadableTrackingBuffer.prototype.enoughLeftFor = function(lengthRequired) {
    return this.buffer.length - this.position >= lengthRequired;
  };
  ReadableTrackingBuffer.prototype.readUInt8 = function() {
    var length, value;
    length = 1;
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return value = this.buffer.readUInt8(this.position - length);
    }
  };
  ReadableTrackingBuffer.prototype.readUInt16LE = function() {
    var length;
    length = 2;
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return this.buffer.readUInt16LE(this.position - length);
    }
  };
  ReadableTrackingBuffer.prototype.readUInt16BE = function() {
    var length;
    length = 2;
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return this.buffer.readUInt16BE(this.position - length);
    }
  };
  ReadableTrackingBuffer.prototype.readUInt32LE = function() {
    var length;
    length = 4;
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return this.buffer.readUInt32LE(this.position - length);
    }
  };
  ReadableTrackingBuffer.prototype.readUInt32BE = function() {
    var length;
    length = 4;
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return this.buffer.readUInt32BE(this.position - length);
    }
  };
  ReadableTrackingBuffer.prototype.readInt8 = function() {
    var length, value;
    length = 1;
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return value = this.buffer.readInt8(this.position - length);
    }
  };
  ReadableTrackingBuffer.prototype.readInt16LE = function() {
    var length;
    length = 2;
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return this.buffer.readInt16LE(this.position - length);
    }
  };
  ReadableTrackingBuffer.prototype.readInt16BE = function() {
    var length;
    length = 2;
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return this.buffer.readInt16BE(this.position - length);
    }
  };
  ReadableTrackingBuffer.prototype.readInt32LE = function() {
    var length;
    length = 4;
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return this.buffer.readInt32LE(this.position - length);
    }
  };
  ReadableTrackingBuffer.prototype.readInt32BE = function() {
    var length;
    length = 4;
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return this.buffer.readInt32BE(this.position - length);
    }
  };
  ReadableTrackingBuffer.prototype.readString = function(length) {
    if (!this.enoughLeftFor(length)) {
      return null;
    } else {
      this.position += length;
      return this.buffer.toString(this.encoding, this.position - length, this.position);
    }
  };
  return ReadableTrackingBuffer;
})();
module.exports = ReadableTrackingBuffer;