var TYPE, parser, sprintf;
TYPE = require('./data-type').TYPE;
sprintf = require('sprintf').sprintf;
parser = function(buffer, position) {
  var c, colName, colNameLength, collation, columnCount, columns, dataLength, error, flags, precision, scale, startPosition, token, type, typeNumber, userType;
  startPosition = position;
  if (buffer.length - position < 2) {
    return false;
  }
  columnCount = buffer.readUInt16LE(position);
  position += 2;
  columns = [];
  for (c = 1; 1 <= columnCount ? c <= columnCount : c >= columnCount; 1 <= columnCount ? c++ : c--) {
    if (buffer.length - position < 4 + 2 + 1) {
      return false;
    }
    userType = buffer.readUInt32LE(position);
    position += 4;
    flags = buffer.readUInt16LE(position);
    position += 2;
    typeNumber = buffer.readUInt8(position);
    type = TYPE[typeNumber];
    position++;
    if (!type) {
      error = sprintf('Unrecognised data type 0x%02X at offset 0x%04X', typeNumber, position - 1);
      break;
    }
    if (type.fixedLength) {
      dataLength = type.dataLength;
    } else if (type.variableLength) {
      if (buffer.length - position < type.dataLengthLength) {
        return false;
      }
      switch (type.dataLengthLength) {
        case 1:
          dataLength = buffer.readUInt8(position);
          break;
        case 2:
          dataLength = buffer.readUInt16LE(position);
          break;
        case 4:
          dataLength = buffer.readUInt32LE(position);
          break;
        default:
          error = "Unrecognised dataLengthLength for type " + type;
          break;
      }
      position += type.dataLengthLength;
    }
    if (type.hasPrecision) {
      precision = buffer.readUInt8(position);
      position++;
    } else {
      precision = void 0;
    }
    if (type.hasScale) {
      scale = buffer.readUInt8(position);
      position++;
    } else {
      scale = void 0;
    }
    if (type.hasCollation) {
      if (buffer.length - position < 5) {
        return false;
      }
      collation = Array.prototype.slice.call(buffer, position, position + 5);
      position += 5;
    } else {
      collation = void 0;
    }
    if (buffer.length - position < 1) {
      return false;
    }
    colNameLength = buffer.readUInt8(position) * 2;
    position++;
    if (buffer.length - position < colNameLength) {
      return false;
    }
    colName = buffer.toString('ucs-2', position, position + colNameLength);
    position += colNameLength;
    columns.push({
      userType: userType,
      flags: flags,
      type: type,
      colName: colName,
      collation: collation,
      precision: precision,
      scale: scale,
      dataLength: dataLength
    });
  }
  if (error) {
    return token = {
      name: 'COLMETADATA',
      error: error
    };
  } else {
    return token = {
      name: 'COLMETADATA',
      length: position - startPosition,
      event: 'columnMetadata',
      columns: columns
    };
  }
};
module.exports = parser;