var EventEmitter, types;
EventEmitter = require('events').EventEmitter;
types = {
  1: {
    name: 'DATABASE',
    event: 'databaseChange'
  },
  2: {
    name: 'LANGUAGE',
    event: 'languageChange'
  },
  3: {
    name: 'CHARSET',
    event: 'charsetChange'
  },
  4: {
    name: 'PACKET_SIZE',
    event: 'packetSizeChange'
  },
  7: {
    name: 'SQL_COLLATION',
    event: 'sqlCollationChange'
  },
  8: {
    name: 'BEGIN_TXN'
  },
  9: {
    name: 'COMMIT_TXN'
  },
  10: {
    name: 'ROLLBACK_TXN'
  },
  17: {
    name: 'TXN_ENDED'
  }
};
module.exports = function(buffer, position) {
  var error, length, newValue, oldValue, token, type, typeNumber, valueLength;
  if (buffer.length - position < 3) {
    return false;
  }
  length = buffer.readUInt16LE(position);
  position += 2;
  if (buffer.length - position < length) {
    return false;
  }
  typeNumber = buffer.readUInt8(position);
  position++;
  type = types[typeNumber];
  if (type) {
    switch (type.name) {
      case 'DATABASE':
      case 'LANGUAGE':
      case 'CHARSET':
      case 'PACKET_SIZE':
        valueLength = buffer.readUInt8(position) * 2;
        position++;
        newValue = buffer.toString('ucs-2', position, position + valueLength);
        position += valueLength;
        valueLength = buffer.readUInt8(position) * 2;
        position++;
        oldValue = buffer.toString('ucs-2', position, position + valueLength);
        position += valueLength;
        break;
      case 'SQL_COLLATION':
        valueLength = buffer.readUInt8(position);
        position++;
        newValue = buffer.slice(position, position + valueLength);
        position += valueLength;
        valueLength = buffer.readUInt8(position);
        position++;
        oldValue = buffer.slice(position, position + valueLength);
        position += valueLength;
        break;
      default:
        error = "Unsupported ENVCHANGE type " + typeNumber + " " + type.name + " at offset " + position;
    }
    if (type.name === 'PACKET_SIZE') {
      newValue = parseInt(newValue);
      oldValue = parseInt(oldValue);
    }
  } else {
    error = "Unsupported ENVCHANGE type " + typeNumber;
  }
  if (error) {
    return token = {
      name: 'ENVCHANGE',
      error: error
    };
  } else {
    return token = {
      name: 'ENVCHANGE',
      length: length + 2,
      type: type.name,
      event: type.event,
      oldValue: oldValue,
      newValue: newValue
    };
  }
};