var NULL, THREE_AND_A_THIRD, TYPE, parser, sprintf;
TYPE = require('./data-type').TYPE;
sprintf = require('sprintf').sprintf;
NULL = (1 << 16) - 1;
THREE_AND_A_THIRD = 3 + (1 / 3);
parser = function(buffer, position, columnsMetaData) {
  var columnMetaData, columns, dataLength, days, error, isNull, milliseconds, minutes, sign, startPosition, threeHundredthsOfSecond, token, type, value, valueHigh, valueLow, _i, _len;
  startPosition = position;
  columns = [];
  for (_i = 0, _len = columnsMetaData.length; _i < _len; _i++) {
    columnMetaData = columnsMetaData[_i];
    isNull = false;
    type = columnMetaData.type;
    switch (type.name) {
      case 'TinyInt':
        if (buffer.length - position < type.dataLength) {
          return false;
        }
        value = buffer.readUInt8(position);
        position += type.dataLength;
        break;
      case 'Int':
        if (buffer.length - position < type.dataLength) {
          return false;
        }
        value = buffer.readInt32LE(position);
        position += type.dataLength;
        break;
      case 'SmallInt':
        if (buffer.length - position < type.dataLength) {
          return false;
        }
        value = buffer.readInt16LE(position);
        position += type.dataLength;
        break;
      case 'IntN':
        if (buffer.length - position < 1) {
          return false;
        }
        dataLength = buffer.readUInt8(position);
        position++;
        if (buffer.length - position < dataLength) {
          return false;
        }
        switch (dataLength) {
          case 0:
            isNull = true;
            break;
          case 1:
            value = buffer.readInt8(position);
            break;
          case 2:
            value = buffer.readInt16LE(position);
            break;
          case 4:
            value = buffer.readInt32LE(position);
        }
        position += dataLength;
        break;
      case 'Bit':
        if (buffer.length - position < type.dataLength) {
          return false;
        }
        value = !!buffer.readUInt8(position);
        position += type.dataLength;
        break;
      case 'BitN':
        if (buffer.length - position < 1) {
          return false;
        }
        dataLength = buffer.readUInt8(position);
        position++;
        if (buffer.length - position < dataLength) {
          return false;
        }
        switch (dataLength) {
          case 0:
            isNull = true;
            break;
          case 1:
            value = !!buffer.readUInt8(position);
        }
        position += dataLength;
        break;
      case 'VarChar':
      case 'Char':
        if (buffer.length - position < 2) {
          return false;
        }
        dataLength = buffer.readUInt16LE(position);
        position += 2;
        if (dataLength === NULL) {
          value = void 0;
          isNull = true;
        } else {
          if (buffer.length - position < dataLength) {
            return false;
          }
          value = buffer.toString('ascii', position, position + dataLength);
          position += dataLength;
        }
        break;
      case 'NVarChar':
      case 'NChar':
        if (buffer.length - position < 2) {
          return false;
        }
        dataLength = buffer.readUInt16LE(position);
        position += 2;
        if (dataLength === NULL) {
          value = void 0;
          isNull = true;
        } else {
          if (buffer.length - position < dataLength) {
            return false;
          }
          value = buffer.toString('ucs-2', position, position + dataLength);
          position += dataLength;
        }
        break;
      case 'SmallDateTime':
        if (buffer.length - position < 4) {
          return false;
        }
        days = buffer.readUInt16LE(position);
        minutes = buffer.readUInt16LE(position + 2);
        value = new Date(1900, 0, 1);
        value.setDate(value.getDate() + days);
        value.setMinutes(value.getMinutes() + minutes);
        position += type.dataLength;
        break;
      case 'DateTime':
        if (buffer.length - position < 8) {
          return false;
        }
        days = buffer.readInt32LE(position);
        threeHundredthsOfSecond = buffer.readUInt32LE(position + 4);
        milliseconds = threeHundredthsOfSecond * THREE_AND_A_THIRD;
        value = new Date(1900, 0, 1);
        value.setDate(value.getDate() + days);
        value.setMilliseconds(value.getMilliseconds() + milliseconds);
        position += type.dataLength;
        break;
      case 'DateTimeN':
        if (buffer.length - position < 1) {
          return false;
        }
        dataLength = buffer.readUInt8(position);
        position++;
        if (buffer.length - position < dataLength) {
          return false;
        }
        switch (dataLength) {
          case 0:
            isNull = true;
            break;
          case 4:
            days = buffer.readUInt16LE(position);
            minutes = buffer.readUInt16LE(position + 2);
            value = new Date(1900, 0, 1);
            value.setDate(value.getDate() + days);
            value.setMinutes(value.getMinutes() + minutes);
            break;
          case 8:
            days = buffer.readInt32LE(position);
            threeHundredthsOfSecond = buffer.readUInt32LE(position + 4);
            milliseconds = threeHundredthsOfSecond * THREE_AND_A_THIRD;
            value = new Date(1900, 0, 1);
            value.setDate(value.getDate() + days);
            value.setMilliseconds(value.getMilliseconds() + milliseconds);
        }
        position += dataLength;
        break;
      case 'NumericN':
        if (buffer.length - position < 1) {
          return false;
        }
        dataLength = buffer.readUInt8(position);
        position++;
        if (dataLength === 0) {
          value = void 0;
          isNull = true;
        } else {
          if (buffer.length - position < dataLength) {
            return false;
          }
          sign = buffer.readUInt8(position) === 1 ? 1 : -1;
          position++;
          switch (dataLength - 1) {
            case 4:
              value = buffer.readUInt32LE(position);
              position += 4;
              break;
            case 8:
              valueLow = buffer.readUInt32LE(position);
              position += 4;
              valueHigh = buffer.readUInt32LE(position);
              position += 4;
              value = valueLow + (0x100000000 * valueHigh);
              break;
            default:
              error = sprintf('Unsupported numeric size %d at offset 0x%04X', dataLength - 1, position);
              break;
          }
          value *= sign;
          value /= Math.pow(10, columnMetaData.scale);
        }
        break;
      default:
        error = sprintf('Unrecognised column type %s at offset 0x%04X', type.name, position - 1);
        break;
    }
    columns.push({
      value: value,
      isNull: isNull,
      metadata: columnMetaData
    });
  }
  columns.byName = function() {
    var byName, column, _j, _len2;
    byName = {};
    for (_j = 0, _len2 = columns.length; _j < _len2; _j++) {
      column = columns[_j];
      byName[column.metadata.colName] = column;
    }
    return byName;
  };
  if (error) {
    return token = {
      name: 'ROW',
      error: error
    };
  } else {
    return token = {
      name: 'ROW',
      length: position - startPosition,
      event: 'row',
      columns: columns
    };
  }
};
module.exports = parser;