var EventEmitter, Parser, TYPE, tokenParsers;
var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
  for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
  function ctor() { this.constructor = child; }
  ctor.prototype = parent.prototype;
  child.prototype = new ctor;
  child.__super__ = parent.prototype;
  return child;
};
require('buffertools');
EventEmitter = require('events').EventEmitter;
TYPE = require('./token').TYPE;
tokenParsers = {};
tokenParsers[TYPE.COLMETADATA] = require('./colmetadata-token-parser');
tokenParsers[TYPE.DONE] = require('./done-token-parser').doneParser;
tokenParsers[TYPE.DONEINPROC] = require('./done-token-parser').doneInProcParser;
tokenParsers[TYPE.DONEPROC] = require('./done-token-parser').doneProcParser;
tokenParsers[TYPE.ENVCHANGE] = require('./env-change-token-parser');
tokenParsers[TYPE.ERROR] = require('./infoerror-token-parser').errorParser;
tokenParsers[TYPE.INFO] = require('./infoerror-token-parser').infoParser;
tokenParsers[TYPE.LOGINACK] = require('./loginack-token-parser');
tokenParsers[TYPE.RETURNSTATUS] = require('./returnstatus-token-parser');
tokenParsers[TYPE.ROW] = require('./row-token-parser');
/*
  Buffers are thrown at the parser (by calling addBuffer).
  Tokens are parsed from the buffer until there are no more tokens in 
  the buffer, or there is just a partial token left.
  If there is a partial token left over, then it is kept until another
  buffer is added, which should contain the remainder of the partial
  token, along with (perhaps) more tokens.
  The partial token and the new buffer are concatenated, and the token
  parsing resumes.
*/
Parser = (function() {
  __extends(Parser, EventEmitter);
  function Parser(debug) {
    this.debug = debug;
    this.buffer = new Buffer(0);
    this.position = 0;
  }
  Parser.prototype.addBuffer = function(buffer) {
    this.buffer = new Buffer(this.buffer.concat(buffer));
    while (this.nextToken()) {
      'NOOP';
    }
    this.buffer = this.buffer.slice(this.position);
    return this.position = 0;
  };
  Parser.prototype.end = function() {
    return this.buffer.length === 0;
  };
  Parser.prototype.nextToken = function() {
    var token, type;
    if (this.position >= this.buffer.length) {
      return false;
    }
    type = this.buffer.readUInt8(this.position);
    if (tokenParsers[type]) {
      token = tokenParsers[type](this.buffer, this.position + 1, this.colMetadata);
      if (token) {
        this.debug.token(token);
        if (!token.error) {
          this.position += 1 + token.length;
          if (token.event) {
            this.emit(token.event, token);
          }
          switch (token.name) {
            case 'COLMETADATA':
              this.colMetadata = token.columns;
          }
          return true;
        } else {
          this.emit('error', token);
          return false;
        }
      } else {
        return false;
      }
    } else {
      return false;
    }
  };
  return Parser;
})();
exports.Parser = Parser;