var Connection, Request, config, fs;
Connection = require('../../lib/connection');
Request = require('../../lib/request');
fs = require('fs');
config = JSON.parse(fs.readFileSync(process.env.HOME + '/.tedious/test-connection.json', 'utf8'));
config.options.debug = {
  data: true,
  payload: true,
  token: true
};
exports.connect = function(test) {
  var connection;
  connection = new Connection(config.server, config.userName, config.password, config.options, function(err, loggedIn) {
    test.ok(!err);
    test.ok(loggedIn);
    test.strictEqual(connection.database(), config.options.database);
    return test.done();
  });
  connection.on('databaseChange', function(database) {
    return test.strictEqual(database, config.options.database);
  });
  return connection.on('debug', function(message) {});
};
exports.execSimpleSql = function(test) {
  var connection, request;
  test.expect(9);
  request = new Request('select 8 as C1', function(err, rowCount) {
    test.ok(!err);
    test.strictEqual(rowCount, 1);
    return test.done();
  });
  request.on('columnMetadata', function(columnsMetadata) {
    return test.strictEqual(columnsMetadata.length, 1);
  });
  request.on('row', function(columns) {
    var byName;
    test.strictEqual(columns.length, 1);
    test.strictEqual(columns[0].value, 8);
    test.strictEqual(columns[0].isNull, false);
    byName = columns.byName();
    return test.strictEqual(byName.C1.value, 8);
  });
  connection = new Connection(config.server, config.userName, config.password, config.options, function(err, loggedIn) {
    test.ok(!err);
    test.ok(loggedIn);
    return connection.execSql(request);
  });
  return connection.on('debug', function(message) {});
};
exports.execSqlWithLotsOfRowsReturned = function(test) {
  var connection, numberOfRows, request, rowsReceived;
  numberOfRows = 1000;
  rowsReceived = 0;
  test.expect(6);
  request = new Request("select top " + numberOfRows + " object_id, name from sys.all_columns", function(err, rowCount) {
    test.ok(!err);
    test.strictEqual(rowCount, numberOfRows);
    test.strictEqual(rowsReceived, numberOfRows);
    return test.done();
  });
  request.on('columnMetadata', function(columnsMetadata) {
    return test.strictEqual(columnsMetadata.length, 2);
  });
  request.on('row', function(columns) {
    return rowsReceived++;
  });
  connection = new Connection(config.server, config.userName, config.password, config.options, function(err, loggedIn) {
    test.ok(!err);
    test.ok(loggedIn);
    return connection.execSql(request);
  });
  return connection.on('debug', function(message) {});
};
exports.execBadSql = function(test) {
  var connection, request;
  test.expect(6);
  request = new Request('select bad syntax here', function(err, rowCount) {
    test.ok(err);
    test.strictEqual(rowCount, void 0);
    return test.done();
  });
  request.on('row', function(columns) {
    return test.ok(false);
  });
  connection = new Connection(config.server, config.userName, config.password, config.options, function(err, loggedIn) {
    test.ok(!err);
    test.ok(loggedIn);
    return connection.execSql(request);
  });
  connection.on('errorMessage', function(error) {
    test.ok(error.message.indexOf('syntax'));
    return test.strictEqual(error.number, 102);
  });
  return connection.on('debug', function(message) {});
};
exports.execSqlProc = function(test) {
  var connection, request, rows;
  rows = 0;
  request = new Request('exec sp_who2', function(err, returnStatus) {
    test.ok(!err);
    test.strictEqual(returnStatus, 0);
    test.ok(rows > 0);
    return test.done();
  });
  request.on('columnMetadata', function(columnsMetadata) {
    return test.strictEqual(columnsMetadata.length, 13);
  });
  request.on('row', function(columns) {
    rows++;
    return test.strictEqual(columns.length, 13);
  });
  connection = new Connection(config.server, config.userName, config.password, config.options, function(err, loggedIn) {
    test.ok(!err);
    test.ok(loggedIn);
    return connection.execSql(request);
  });
  return connection.on('debug', function(message) {});
};
exports.badCredentials = function(test) {
  var connection;
  test.expect(4);
  connection = new Connection(config.server, config.userName, 'bad-password', config.options, function(err, loggedIn) {
    test.ok(err);
    test.ok(!loggedIn);
    return test.done();
  });
  connection.on('errorMessage', function(error) {
    test.ok(error.message.indexOf('failed'));
    return test.strictEqual(error.number, 18456);
  });
  return connection.on('debug', function(message) {});
};
exports.badServer = function(test) {
  var connection;
  connection = new Connection('bad-server', config.userName, config.password, config.options, function(err, loggedIn) {
    return test.ok(false);
  });
  connection.on('fatal', function(error) {
    return test.done();
  });
  return connection.on('debug', function(message) {});
};
exports.badPort = function(test) {
  var connection;
  config.options.port = -1;
  connection = new Connection(config.server, config.userName, config.password, config.options, function(err, loggedIn) {
    return test.ok(false);
  });
  connection.on('fatal', function(error) {
    return test.done();
  });
  return connection.on('debug', function(message) {});
};