var Connection, Request, config, execSql, fs;
Connection = require('../../lib/connection');
Request = require('../../lib/request');
fs = require('fs');
config = JSON.parse(fs.readFileSync(process.env.HOME + '/.tedious/test-connection.json', 'utf8'));
config.options.debug = {
  data: true,
  payload: true,
  token: true
};
exports["null"] = function(test) {
  return execSql(test, 'select null', null);
};
exports.tinyint = function(test) {
  return execSql(test, 'select cast(8 as tinyint)', 8);
};
exports.tinyintNull = function(test) {
  return execSql(test, 'select cast(null as tinyint)', null);
};
exports.smallint = function(test) {
  return execSql(test, 'select cast(8 as smallint)', 8);
};
exports.smallintNull = function(test) {
  return execSql(test, 'select cast(null as smallint)', null);
};
exports.int = function(test) {
  return execSql(test, 'select cast(8 as int)', 8);
};
exports.intNull = function(test) {
  return execSql(test, 'select cast(null as int)', null);
};
exports.bitFalse = function(test) {
  return execSql(test, "select cast('false' as bit)", false);
};
exports.bitTrue = function(test) {
  return execSql(test, "select cast('true' as bit)", true);
};
exports.bitNull = function(test) {
  return execSql(test, "select cast(null as bit)", null);
};
exports.datetime = function(test) {
  return execSql(test, "select cast('2011-12-4 10:04:23' as datetime)", new Date('December 4, 2011 10:04:23'));
};
exports.datetimeNull = function(test) {
  return execSql(test, "select cast(null as datetime)", null);
};
exports.numericSmallValue = function(test) {
  return execSql(test, "select cast(9.3 as numeric(3,2))", 9.3);
};
exports.numericLargeValue = function(test) {
  return execSql(test, "select cast(9876543.3456 as numeric(12,5))", 9876543.3456);
};
exports.numericNull = function(test) {
  return execSql(test, "select cast(null as numeric(3,2))", null);
};
exports.varchar = function(test) {
  return execSql(test, "select cast('abc' as varchar(10))", 'abc');
};
exports.varcharNull = function(test) {
  return execSql(test, "select cast(null as varchar(10))", null);
};
exports.nvarchar = function(test) {
  return execSql(test, "select cast('abc' as nvarchar(10))", 'abc');
};
exports.nvarcharNull = function(test) {
  return execSql(test, "select cast(null as nvarchar(10))", null);
};
exports.char = function(test) {
  return execSql(test, "select cast('abc' as char(5))", 'abc  ');
};
exports.charNull = function(test) {
  return execSql(test, "select cast(null as char(5))", null);
};
exports.nchar = function(test) {
  return execSql(test, "select cast('abc' as nchar(5))", 'abc  ');
};
exports.ncharNull = function(test) {
  return execSql(test, "select cast(null as nchar(5))", null);
};
execSql = function(test, sql, expectedValue) {
  var connection, request;
  test.expect(3);
  request = new Request(sql, function(err, rowCount) {
    test.ok(!err);
    return test.done();
  });
  request.on('row', function(columns) {
    if (expectedValue === null) {
      return test.ok(columns[0].isNull);
    } else if (expectedValue instanceof Date) {
      return test.strictEqual(columns[0].value.getTime(), expectedValue.getTime());
    } else {
      return test.strictEqual(columns[0].value, expectedValue);
    }
  });
  connection = new Connection(config.server, config.userName, config.password, config.options, function(err, loggedIn) {
    test.ok(!err);
    return connection.execSql(request);
  });
  connection.on('error', function(message) {
    return console.log(message);
  });
  return connection.on('debug', function(message) {});
};