var Connection, Debug, EventEmitter, MessageIO, Packet, packetType;
var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
  for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
  function ctor() { this.constructor = child; }
  ctor.prototype = parent.prototype;
  child.prototype = new ctor;
  child.__super__ = parent.prototype;
  return child;
};
Debug = require('../../lib/debug');
EventEmitter = require('events').EventEmitter;
MessageIO = require('../../lib/message-io');
Packet = require('../../lib/packet').Packet;
require('buffertools');
Connection = (function() {
  __extends(Connection, EventEmitter);
  function Connection() {
    Connection.__super__.constructor.apply(this, arguments);
  }
  Connection.prototype.setTimeout = function() {};
  Connection.prototype.connect = function() {};
  Connection.prototype.write = function(data) {
    var packet;
    packet = new Packet(data);
    return this.emit('packet', packet);
  };
  return Connection;
})();
packetType = 2;
exports.sendSmallerThanOnePacket = function(test) {
  var connection, io, payload;
  payload = new Buffer([1, 2, 3]);
  connection = new Connection();
  connection.on('packet', function(packet) {
    test.ok(packet.last());
    test.strictEqual(packet.type(), packetType);
    test.ok(packet.data().equals(payload));
    return test.done();
  });
  io = new MessageIO(connection, new Debug());
  io.packetSize(4);
  return io.sendMessage(packetType, payload);
};
exports.sendExactlyPacket = function(test) {
  var connection, io, payload;
  payload = new Buffer([1, 2, 3, 4]);
  connection = new Connection();
  connection.on('packet', function(packet) {
    test.ok(packet.last());
    test.strictEqual(packet.type(), packetType);
    test.ok(packet.data().equals(payload));
    return test.done();
  });
  io = new MessageIO(connection, new Debug());
  io.packetSize(4);
  return io.sendMessage(packetType, payload);
};
exports.sendOneLongerThanPacket = function(test) {
  var connection, io, packetNumber, payload;
  payload = new Buffer([1, 2, 3, 4, 5]);
  packetNumber = 0;
  connection = new Connection();
  connection.on('packet', function(packet) {
    packetNumber++;
    test.strictEqual(packet.type(), packetType);
    switch (packetNumber) {
      case 1:
        test.ok(!packet.last());
        test.strictEqual(packet.packetId(), packetNumber - 1);
        return test.ok(packet.data().equals(new Buffer([1, 2, 3, 4])));
      case 2:
        test.ok(packet.last());
        test.strictEqual(packet.packetId(), packetNumber - 1);
        test.ok(packet.data().equals(new Buffer([5])));
        return test.done();
    }
  });
  io = new MessageIO(connection, new Debug());
  io.packetSize(4);
  return io.sendMessage(packetType, payload);
};
exports.receiveOnePacket = function(test) {
  var connection, io, packet, payload;
  payload = new Buffer([1, 2, 3]);
  connection = new Connection();
  io = new MessageIO(connection, new Debug());
  io.on('packet', function(packet) {
    test.strictEqual(packet.type(), packetType);
    test.ok(packet.data().equals(payload));
    return test.done();
  });
  packet = new Packet(packetType);
  packet.last(true);
  packet.addData(payload);
  return connection.emit('data', packet.buffer);
};
exports.receiveOnePacketInTwoChunks = function(test) {
  var connection, io, packet, payload;
  payload = new Buffer([1, 2, 3]);
  connection = new Connection();
  io = new MessageIO(connection, new Debug());
  io.on('packet', function(packet) {
    test.strictEqual(packet.type(), packetType);
    test.ok(packet.data().equals(payload));
    return test.done();
  });
  packet = new Packet(packetType);
  packet.last(true);
  packet.addData(payload);
  connection.emit('data', packet.buffer.slice(0, 4));
  return connection.emit('data', packet.buffer.slice(4));
};
exports.receiveTwoPackets = function(test) {
  var connection, io, packet, payload, payload1, payload2, receivedPacketCount;
  payload = new Buffer([1, 2, 3]);
  payload1 = payload.slice(0, 2);
  payload2 = payload.slice(2, 3);
  connection = new Connection();
  receivedPacketCount = 0;
  io = new MessageIO(connection, new Debug());
  io.on('packet', function(packet) {
    receivedPacketCount++;
    test.strictEqual(packet.type(), packetType);
    switch (receivedPacketCount) {
      case 1:
        return test.ok(packet.data().equals(payload1));
      case 2:
        test.ok(packet.data().equals(payload2));
        return test.done();
    }
  });
  packet = new Packet(packetType);
  packet.addData(payload1);
  connection.emit('data', packet.buffer);
  packet = new Packet(packetType);
  packet.last(true);
  packet.addData(payload2);
  return connection.emit('data', packet.buffer);
};
exports.receiveTwoPacketsWithChunkSpanningPackets = function(test) {
  var connection, io, packet1, packet2, payload, payload1, payload2, receivedPacketCount;
  payload = new Buffer([1, 2, 3, 4]);
  payload1 = payload.slice(0, 2);
  payload2 = payload.slice(2, 4);
  connection = new Connection();
  receivedPacketCount = 0;
  io = new MessageIO(connection, new Debug());
  io.on('packet', function(packet) {
    receivedPacketCount++;
    test.strictEqual(packet.type(), packetType);
    switch (receivedPacketCount) {
      case 1:
        return test.ok(packet.data().equals(payload1));
      case 2:
        test.ok(packet.data().equals(payload2));
        return test.done();
    }
  });
  packet1 = new Packet(packetType);
  packet1.addData(payload.slice(0, 2));
  packet2 = new Packet(packetType);
  packet2.last(true);
  packet2.addData(payload.slice(2, 4));
  connection.emit('data', packet1.buffer.slice(0, 6));
  connection.emit('data', packet1.buffer.slice(6).concat(packet2.buffer.slice(0, 4)));
  return connection.emit('data', packet2.buffer.slice(4));
};
exports.receiveMultiplePacketsWithMoreThanOnePacketFromOneChunk = function(test) {
  var allData, connection, data1, data2, io, packet1, packet2, packet3, payload, payload1, payload2, payload3, receivedPacketCount;
  payload = new Buffer([1, 2, 3, 4, 5, 6]);
  payload1 = payload.slice(0, 2);
  payload2 = payload.slice(2, 4);
  payload3 = payload.slice(4, 6);
  connection = new Connection();
  receivedPacketCount = 0;
  io = new MessageIO(connection, new Debug());
  io.on('packet', function(packet) {
    receivedPacketCount++;
    test.strictEqual(packet.type(), packetType);
    switch (receivedPacketCount) {
      case 1:
        return test.ok(packet.data().equals(payload1));
      case 2:
        return test.ok(packet.data().equals(payload2));
      case 3:
        test.ok(packet.data().equals(payload3));
        return test.done();
    }
  });
  packet1 = new Packet(packetType);
  packet1.addData(payload.slice(0, 2));
  packet2 = new Packet(packetType);
  packet2.addData(payload.slice(2, 4));
  packet3 = new Packet(packetType);
  packet3.last(true);
  packet3.addData(payload.slice(4, 6));
  allData = new Buffer(packet1.buffer.concat(packet2.buffer, packet3.buffer));
  data1 = allData.slice(0, 5);
  data2 = allData.slice(5);
  connection.emit('data', data1);
  return connection.emit('data', data2);
};