var TYPE, WritableBuffer, dataTypeByName, parser;
parser = require('../../../lib/token/row-token-parser');
TYPE = require('../../../lib/token/token').TYPE;
dataTypeByName = require('../../../lib/token/data-type').typeByName;
WritableBuffer = require('../../../lib/tracking-buffer/tracking-buffer').WritableTrackingBuffer;
module.exports.int = function(test) {
  var buffer, colMetaData, token, value;
  colMetaData = [
    {
      type: dataTypeByName.Int
    }
  ];
  value = 3;
  buffer = new WritableBuffer(0);
  buffer.writeUInt8(TYPE.ROW);
  buffer.writeUInt32LE(value);
  buffer = buffer.data;
  token = parser(buffer, 1, colMetaData);
  test.strictEqual(token.length, buffer.length - 1);
  test.strictEqual(token.columns.length, 1);
  test.ok(!token.columns[0].isNull);
  test.strictEqual(token.columns[0].value, value);
  test.strictEqual(token.columns[0].metadata, colMetaData[0]);
  return test.done();
};
module.exports.varChar = function(test) {
  var buffer, colMetaData, token, value;
  colMetaData = [
    {
      type: dataTypeByName.VarChar
    }
  ];
  value = 'abc';
  buffer = new WritableBuffer(0, 'ascii');
  buffer.writeUInt8(TYPE.ROW);
  buffer.writeUInt16LE(value.length);
  buffer.writeString(value);
  buffer = buffer.data;
  token = parser(buffer, 1, colMetaData);
  test.strictEqual(token.length, buffer.length - 1);
  test.strictEqual(token.columns.length, 1);
  test.ok(!token.columns[0].isNull);
  test.strictEqual(token.columns[0].value, value);
  test.strictEqual(token.columns[0].metadata, colMetaData[0]);
  return test.done();
};
module.exports.nVarChar = function(test) {
  var buffer, colMetaData, token, value;
  colMetaData = [
    {
      type: dataTypeByName.NVarChar
    }
  ];
  value = 'abc';
  buffer = new WritableBuffer(0, 'ucs2');
  buffer.writeUInt8(TYPE.ROW);
  buffer.writeUInt16LE(value.length * 2);
  buffer.writeString(value);
  buffer = buffer.data;
  token = parser(buffer, 1, colMetaData);
  test.strictEqual(token.length, buffer.length - 1);
  test.strictEqual(token.columns.length, 1);
  test.ok(!token.columns[0].isNull);
  test.strictEqual(token.columns[0].value, value);
  test.strictEqual(token.columns[0].metadata, colMetaData[0]);
  return test.done();
};
module.exports.intN = function(test) {
  var buffer, colMetaData, token;
  colMetaData = [
    {
      type: dataTypeByName.IntN
    }
  ];
  buffer = new WritableBuffer(0);
  buffer.writeUInt8(TYPE.ROW);
  buffer.writeUInt8(0);
  buffer = buffer.data;
  token = parser(buffer, 1, colMetaData);
  test.strictEqual(token.length, buffer.length - 1);
  test.strictEqual(token.columns.length, 1);
  test.ok(!token.columns[0].value);
  test.ok(token.columns[0].isNull);
  return test.done();
};
module.exports.datetime = function(test) {
  var buffer, colMetaData, days, threeHundredthsOfSecond, token;
  colMetaData = [
    {
      type: dataTypeByName.DateTime
    }
  ];
  days = 2;
  threeHundredthsOfSecond = 45 * 300;
  buffer = new WritableBuffer(0);
  buffer.writeUInt8(TYPE.ROW);
  buffer.writeInt32LE(days);
  buffer.writeUInt32LE(threeHundredthsOfSecond);
  buffer = buffer.data;
  token = parser(buffer, 1, colMetaData);
  test.strictEqual(token.length, buffer.length - 1);
  test.strictEqual(token.columns.length, 1);
  test.ok(!token.columns[0].isNull);
  test.strictEqual(token.columns[0].value.getTime(), new Date('January 3, 1900 00:00:45').getTime());
  return test.done();
};
module.exports.datetimeN = function(test) {
  var buffer, colMetaData, token;
  colMetaData = [
    {
      type: dataTypeByName.DateTimeN
    }
  ];
  buffer = new WritableBuffer(0);
  buffer.writeUInt8(TYPE.ROW);
  buffer.writeUInt8(0);
  buffer = buffer.data;
  token = parser(buffer, 1, colMetaData);
  test.strictEqual(token.length, buffer.length - 1);
  test.strictEqual(token.columns.length, 1);
  test.ok(!token.columns[0].value);
  test.ok(token.columns[0].isNull);
  return test.done();
};
module.exports.numeric = function(test) {
  var buffer, colMetaData, token, value;
  colMetaData = [
    {
      type: dataTypeByName.NumericN,
      precision: 3,
      scale: 1
    }
  ];
  value = -9.3;
  buffer = new WritableBuffer(0);
  buffer.writeUInt8(TYPE.ROW);
  buffer.writeUInt8(5);
  buffer.writeUInt8(0);
  buffer.writeUInt32LE(93);
  buffer = buffer.data;
  token = parser(buffer, 1, colMetaData);
  test.strictEqual(token.length, buffer.length - 1);
  test.strictEqual(token.columns.length, 1);
  test.ok(!token.columns[0].isNull);
  test.strictEqual(token.columns[0].value, value);
  return test.done();
};
module.exports.numericN = function(test) {
  var buffer, colMetaData, token;
  colMetaData = [
    {
      type: dataTypeByName.NumericN,
      precision: 3,
      scale: 1
    }
  ];
  buffer = new WritableBuffer(0);
  buffer.writeUInt8(TYPE.ROW);
  buffer.writeUInt8(0);
  buffer = buffer.data;
  token = parser(buffer, 1, colMetaData);
  test.strictEqual(token.length, buffer.length - 1);
  test.strictEqual(token.columns.length, 1);
  test.ok(!token.columns[0].value);
  test.ok(token.columns[0].isNull);
  return test.done();
};