var TrackingBuffer;
require('buffertools');
TrackingBuffer = require('../../../lib/tracking-buffer/tracking-buffer').ReadableTrackingBuffer;
exports.createNoArgs = function(test) {
  var buffer;
  buffer = new TrackingBuffer();
  test.strictEqual(buffer.buffer.length, 0);
  test.strictEqual(buffer.encoding, 'utf8');
  return test.done();
};
exports.createWithBuffer = function(test) {
  var buffer, inputBuffer;
  inputBuffer = new Buffer([1, 2, 3]);
  buffer = new TrackingBuffer(inputBuffer);
  test.strictEqual(buffer.buffer, inputBuffer);
  test.strictEqual(buffer.encoding, 'utf8');
  return test.done();
};
exports.createWithEncoding = function(test) {
  var buffer, inputBuffer;
  inputBuffer = new Buffer([1, 2, 3]);
  buffer = new TrackingBuffer(inputBuffer, 'ucs2');
  test.strictEqual(buffer.buffer, inputBuffer);
  test.strictEqual(buffer.encoding, 'ucs2');
  return test.done();
};
exports.readUnsignedInt = function(test) {
  var buffer, data;
  data = new Buffer([0x01, 0x02, 0x00, 0x00, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05]);
  buffer = new TrackingBuffer(data.slice(0, 2));
  test.strictEqual(buffer.readUInt8(), 1);
  test.strictEqual(buffer.readUInt16LE(), null);
  buffer.add(data.slice(2, 7));
  test.strictEqual(buffer.readUInt16LE(), 2);
  test.strictEqual(buffer.readUInt16BE(), 3);
  test.strictEqual(buffer.readUInt32LE(), null);
  buffer.add(data.slice(7));
  test.strictEqual(buffer.readUInt32LE(), 4);
  test.strictEqual(buffer.readUInt32BE(), 5);
  return test.done();
};
exports.readSignedInt = function(test) {
  var buffer, data;
  data = new Buffer([0xFF, 0xFE, 0xFF, 0xFF, 0xFD, 0xFC, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFB]);
  buffer = new TrackingBuffer(data.slice(0, 2));
  test.strictEqual(buffer.readInt8(), -1);
  test.strictEqual(buffer.readInt16LE(), null);
  buffer.add(data.slice(2, 7));
  test.strictEqual(buffer.readInt16LE(), -2);
  test.strictEqual(buffer.readInt16BE(), -3);
  test.strictEqual(buffer.readInt32LE(), null);
  buffer.add(data.slice(7));
  test.strictEqual(buffer.readInt32LE(), -4);
  test.strictEqual(buffer.readInt32BE(), -5);
  return test.done();
};
exports.readString = function(test) {
  var buffer, data;
  data = new Buffer([0x61, 0x00, 0x62, 0x00, 0x63, 0x00]);
  buffer = new TrackingBuffer(data.slice(0, 2), 'ucs2');
  test.strictEqual(buffer.readString(data.length), null);
  buffer.add(data.slice(2));
  test.strictEqual(buffer.readString(data.length), 'abc');
  return test.done();
};