var ENCRYPT, MARS, PreloginPayload, SUBBUILD, TOKEN, VERSION, buildBuffer, encryptByValue, marsByValue, name, sprintf, value;
buildBuffer = require('./build-buffer');
sprintf = require('sprintf').sprintf;
VERSION = 0x000000001;
SUBBUILD = 0x0001;
TOKEN = {
  VERSION: 0x00,
  ENCRYPTION: 0x01,
  INSTOPT: 0x02,
  THREADID: 0x03,
  MARS: 0x04,
  TERMINATOR: 0xFF
};
ENCRYPT = {
  OFF: 0x00,
  ON: 0x01,
  NOT_SUP: 0x02,
  REQ: 0x03
};
encryptByValue = {};
for (name in ENCRYPT) {
  value = ENCRYPT[name];
  encryptByValue[value] = name;
}
MARS = {
  OFF: 0x00,
  ON: 0x01
};
marsByValue = {};
for (name in MARS) {
  value = MARS[name];
  marsByValue[value] = name;
}
/*
  s2.2.6.4
*/
PreloginPayload = (function() {
  function PreloginPayload(buffer) {
    if (buffer instanceof Buffer) {
      this.data = buffer;
    } else {
      this.createOptions();
    }
    this.extractOptions();
  }
  PreloginPayload.prototype.createOptions = function() {
    var length, option, optionDataOffset, optionOffset, options, _i, _j, _len, _len2;
    options = [this.createVersionOption(), this.createEncryptionOption(), this.createInstanceOption(), this.createThreadIdOption(), this.createMarsOption()];
    length = 0;
    for (_i = 0, _len = options.length; _i < _len; _i++) {
      option = options[_i];
      length += 5 + option.data.length;
    }
    length++;
    this.data = new Buffer(length);
    optionOffset = 0;
    optionDataOffset = 5 * options.length + 1;
    for (_j = 0, _len2 = options.length; _j < _len2; _j++) {
      option = options[_j];
      this.data.writeUInt8(option.token, optionOffset + 0);
      this.data.writeUInt16BE(optionDataOffset, optionOffset + 1);
      this.data.writeUInt16BE(option.data.length, optionOffset + 3);
      optionOffset += 5;
      option.data.copy(this.data, optionDataOffset);
      optionDataOffset += option.data.length;
    }
    return this.data.writeUInt8(TOKEN.TERMINATOR, optionOffset);
  };
  PreloginPayload.prototype.createVersionOption = function() {
    return {
      token: TOKEN.VERSION,
      data: buildBuffer('U32', VERSION, 'U16', SUBBUILD)
    };
  };
  PreloginPayload.prototype.createEncryptionOption = function() {
    return {
      token: TOKEN.ENCRYPTION,
      data: buildBuffer('U8', ENCRYPT.NOT_SUP)
    };
  };
  PreloginPayload.prototype.createInstanceOption = function() {
    return {
      token: TOKEN.INSTOPT,
      data: buildBuffer('U8', 0x00)
    };
  };
  PreloginPayload.prototype.createThreadIdOption = function() {
    return {
      token: TOKEN.THREADID,
      data: buildBuffer('U32', 0x00)
    };
  };
  PreloginPayload.prototype.createMarsOption = function() {
    return {
      token: TOKEN.MARS,
      data: buildBuffer('U8', MARS.OFF)
    };
  };
  PreloginPayload.prototype.extractOptions = function() {
    var dataLength, dataOffset, offset, _results;
    offset = 0;
    _results = [];
    while (this.data[offset] !== TOKEN.TERMINATOR) {
      dataOffset = this.data.readUInt16BE(offset + 1);
      dataLength = this.data.readUInt16BE(offset + 3);
      switch (this.data[offset]) {
        case TOKEN.VERSION:
          this.extractVersion(dataOffset);
          break;
        case TOKEN.ENCRYPTION:
          this.extractEncryption(dataOffset);
          break;
        case TOKEN.INSTOPT:
          this.extractInstance(dataOffset);
          break;
        case TOKEN.THREADID:
          if (dataLength > 0) {
            this.extractThreadId(dataOffset);
          }
          break;
        case TOKEN.MARS:
          this.extractMars(dataOffset);
      }
      offset += 5;
      _results.push(dataOffset += dataLength);
    }
    return _results;
  };
  PreloginPayload.prototype.extractVersion = function(offset) {
    return this.version = {
      major: this.data.readUInt8(offset + 0),
      minor: this.data.readUInt8(offset + 1),
      patch: this.data.readUInt8(offset + 2),
      trivial: this.data.readUInt8(offset + 3),
      subbuild: this.data.readUInt16BE(offset + 4)
    };
  };
  PreloginPayload.prototype.extractEncryption = function(offset) {
    this.encryption = this.data.readUInt8(offset);
    return this.encryptionString = encryptByValue[this.encryption];
  };
  PreloginPayload.prototype.extractInstance = function(offset) {
    return this.instance = this.data.readUInt8(offset);
  };
  PreloginPayload.prototype.extractThreadId = function(offset) {
    return this.threadId = this.data.readUInt32BE(offset);
  };
  PreloginPayload.prototype.extractMars = function(offset) {
    this.mars = this.data.readUInt8(offset);
    return this.marsString = marsByValue[this.mars];
  };
  PreloginPayload.prototype.toString = function(indent) {
    indent || (indent = '');
    return indent + 'PreLogin - ' + sprintf('version:%d.%d.%d.%d %d, encryption:0x%02X(%s), instopt:0x%02X, threadId:0x%08X, mars:0x%02X(%s)', this.version.major, this.version.minor, this.version.patch, this.version.trivial, this.version.subbuild, this.encryption, this.encryptionString, this.instance, this.threadId ? this.threadId : 0, this.mars, this.marsString);
  };
  return PreloginPayload;
})();
module.exports = PreloginPayload;